@@ -6,12 +6,13 @@ use strict;
 use warnings;
 
 use File::Spec;
+use Scalar::Util qw( blessed );
 
 use overload
     '""' => \&as_string,
     fallback => 1;
 
-our $VERSION = '1.1901';
+our $VERSION = '1.21';
 
 
 sub new
@@ -46,6 +47,11 @@ sub _record_caller_data
             do { package DB; @DB::args = (); caller($x++) } )
     {
         my @a = @DB::args;
+
+        if ( $self->{no_refs} ) {
+            @a = map { ref $_ ? $self->_ref_to_string($_) : $_ } @a;
+        }
+
         push @{ $self->{raw} },
             { caller => \@c,
               args   => \@a,
@@ -53,10 +59,67 @@ sub _record_caller_data
     }
 }
 
+sub _ref_to_string
+{
+    my $self = shift;
+    my $ref  = shift;
+
+    return overload::StrVal($ref)
+        if blessed $ref && $ref->isa('Exception::Class::Base');
+
+    return overload::StrVal($ref) unless $self->{respect_overload};
+
+    local $@;
+    local $SIG{__DIE__};
+
+    my $str = eval { $ref . '' };
+
+    return $@ ? overload::StrVal($ref) : $str;
+}
+
+sub _ecb_hack
+{
+    my $self = shift;
+    my $ref  = shift;
+
+    # This avoids a loop between Exception::Class::Base and this module
+    if ( $ref->can('show_trace') )
+    {
+        my $t = $ref->show_trace;
+        $ref->show_trace(0);
+        my $s = "$ref";
+        $ref->show_trace($t);
+
+        return $s;
+    }
+    else
+    {
+        # hack but should work with older
+        # versions of E::C::B
+        return $ref->{message};
+    }
+}
+
 sub _make_frames
 {
     my $self = shift;
 
+    my $filter = $self->_make_frame_filter;
+
+    my $raw = delete $self->{raw};
+    for my $r ( @{$raw} )
+    {
+        next unless $filter->($r);
+
+        $self->_add_frame( $r->{caller}, $r->{args} );
+    }
+}
+
+my $default_filter = sub { 1 };
+sub _make_frame_filter
+{
+    my $self = shift;
+
     my (@i_pack_re, %i_class);
     if ( $self->{ignore_package} )
     {
@@ -66,23 +129,29 @@ sub _make_frames
         @i_pack_re = map { ref $_ ? $_ : qr/^\Q$_\E$/ } @{ $self->{ignore_package} };
     }
 
+    my $p = __PACKAGE__;
+    push @i_pack_re, qr/^\Q$p\E$/;
+
     if ( $self->{ignore_class} )
     {
         $self->{ignore_class} = [ $self->{ignore_class} ] unless ref $self->{ignore_class};
         %i_class = map {$_ => 1} @{ $self->{ignore_class} };
     }
 
-    my $p = __PACKAGE__;
-    push @i_pack_re, qr/^\Q$p\E$/;
+    my $user_filter = $self->{frame_filter};
 
-    my $raw = delete $self->{raw};
-    for my $r ( @{$raw} )
+    return sub
     {
-        next if grep { $r->{caller}[0] =~ /$_/ } @i_pack_re;
-        next if grep { $r->{caller}[0]->isa($_) } keys %i_class;
+        return 0 if grep { $_[0]{caller}[0] =~ /$_/ } @i_pack_re;
+        return 0 if grep { $_[0]{caller}[0]->isa($_) } keys %i_class;
 
-        $self->_add_frame( $r->{caller}, $r->{args} );
-    }
+        if ( $user_filter )
+        {
+            return $user_filter->( $_[0] );
+        }
+
+        return 1;
+    };
 }
 
 sub _add_frame
@@ -96,34 +165,6 @@ sub _add_frame
 
     if ( $self->{no_refs} )
     {
-        for ( grep { ref } @{$args} )
-        {
-            # I can't remember what this is about but I think
-            # it must be to avoid a loop between between
-            # Exception::Class and this module.
-            if ( UNIVERSAL::isa( $_, 'Exception::Class::Base' ) )
-            {
-                $_ = do { if ( $_->can('show_trace') )
-                          {
-                              my $t = $_->show_trace;
-                              $_->show_trace(0);
-                              my $s = "$_";
-                              $_->show_trace($t);
-                              $s;
-                          }
-                          else
-                          {
-                              # hack but should work with older
-                              # versions of E::C::B
-                              $_->{message};
-                          }
-                        };
-            }
-            else
-            {
-                $_ = $self->_ref_as_string($_);
-            }
-        }
     }
 
     push @{ $self->{frames} },
@@ -131,15 +172,6 @@ sub _add_frame
                                      $self->{respect_overload}, $self->{max_arg_length} );
 }
 
-sub _ref_as_string
-{
-    my $self = shift;
-
-    return overload::StrVal($_[0]) unless $self->{respect_overload};
-    # force stringification
-    return $_[0] . '';
-}
-
 sub next_frame
 {
     my $self = shift;
@@ -329,7 +361,7 @@ sub as_string
                 $_ = "undef", next unless defined $_;
 
                 # hack!
-                $_ = $self->Devel::StackTrace::_ref_as_string($_)
+                $_ = $self->Devel::StackTrace::_ref_to_string($_)
                     if ref $_;
 
                 eval
@@ -440,6 +472,23 @@ Takes the following parameters:
 
 =over 8
 
+=item * frame_filter => $sub
+
+By default, Devel::StackTrace will include all stack frames before the
+call to its its constructor.
+
+However, you may want to filter out some frames with more granularity
+than 'ignore_package' or 'ignore_class' allow.
+
+You can provide a subroutine which is called with the raw frame data
+for each frame. This is a hash reference with two keys, "caller", and
+"args", both of which are array references. The "caller" key is the
+raw data as returned by Perl's C<caller()> function, and the "args"
+key are the subroutine arguments found in C<@DB::args>.
+
+The filter should return true if the frame should be included, or
+false if it should be skipped.
+
 =item * ignore_package => $package_name OR \@package_names
 
 Any frames where the package is one of these packages will not be on
